package controllers

import (
	"encoding/json"
	"fmt"
	"slices"

	oclib "cloud.o-forge.io/core/oc-lib"
	"cloud.o-forge.io/core/oc-lib/tools"
	beego "github.com/beego/beego/v2/server/web"
)

// Operations about workspace
type CollaborativeAreaController struct {
	beego.Controller
}

var paths = map[tools.DataType]map[tools.METHOD]string{ // paths used to call other OC services
	tools.COLLABORATIVE_AREA: {
		tools.DELETE: "/collaborative_area/:id?is_remote=true",
		tools.POST:   "/collaborative_area/?is_remote=true",
	},
	tools.WORKSPACE: {
		tools.DELETE: "/:id?is_remote=true",
		tools.POST:   "/?is_remote=true",
	},
	tools.WORKFLOW: {
		tools.DELETE: "/:id?is_remote=true",
		tools.POST:   "/?is_remote=true",
	},
	tools.PEER: {
		tools.POST: "/status/",
	},
}

// @Title Search
// @Description search shared workspace
// @Param	search		path 	string	true		"the word search you want to get"
// @Success 200 {shared workspace} models.shared_workspace
// @router /search/:search [get]
func (o *CollaborativeAreaController) Search() {
	// store and return Id or post with UUID
	search := o.Ctx.Input.Param(":search")
	o.Data["json"] = oclib.Search(nil, search, oclib.LibDataEnum(tools.COLLABORATIVE_AREA))
	o.ServeJSON()
}

// @Title Update
// @Description create shared workspaces
// @Param	id		path 	string	true		"the shared workspace id you want to get"
// @Param	body		body 	models.workspace	true		"The shared workspace content"
// @Success 200 {shared workspace} models.shared_workspace
// @router /:id [put]
func (o *CollaborativeAreaController) Put() {
	// store and return Id or post with UUID
	var res map[string]interface{}
	id := o.Ctx.Input.Param(":id")
	json.Unmarshal(o.Ctx.Input.CopyBody(10000), &res)
	caller := tools.NewHTTPCaller(paths) // caller used to send to peers
	caller.Disabled = oclib.IsQueryParamsEquals(o.Ctx.Input, "is_remote", true)
	fmt.Println("UPDATE", res)
	o.Data["json"] = oclib.UpdateOne(oclib.LibDataEnum(tools.COLLABORATIVE_AREA), res, id, caller)
	o.ServeJSON()
}

// @Title Create
// @Description create shared workspace
// @Param	data		body 	json	true		"body for data content (Json format)"
// @Success 200 {shared workspace} models.shared_workspace
// @router / [post]
func (o *CollaborativeAreaController) Post() {
	caller := tools.NewHTTPCaller(paths) // caller used to send to peers
	caller.Disabled = oclib.IsQueryParamsEquals(o.Ctx.Input, "is_remote", true)
	var res map[string]interface{}
	json.Unmarshal(o.Ctx.Input.CopyBody(10000), &res)
	o.Data["json"] = oclib.StoreOne(oclib.LibDataEnum(tools.COLLABORATIVE_AREA), res, caller)
	o.ServeJSON()
}

// @Title GetAll
// @Description find shared workspace by id
// @Success 200 {shared_workspace} models.shared_workspace
// @router / [get]
func (o *CollaborativeAreaController) GetAll() {
	o.Data["json"] = oclib.LoadAll(oclib.LibDataEnum(tools.COLLABORATIVE_AREA))
	o.ServeJSON()
}

// @Title Get
// @Description find shared workspace by id
// @Param	id		path 	string	true		"the id you want to get"
// @Success 200 {shared workspace} models.shared_workspace
// @router /:id [get]
func (o *CollaborativeAreaController) Get() {
	id := o.Ctx.Input.Param(":id")
	o.Data["json"] = oclib.LoadOne(oclib.LibDataEnum(tools.COLLABORATIVE_AREA), id)
	o.ServeJSON()
}

// @Title Add Workspace
// @Description find shared workspace by id
// @Param	id		path 	string	true		"the id you want to get"
// @Param	id2		path 	string	true		"the id you want to add"
// @Success 200 {shared workspace} models.shared_workspace
// @router /:id/workspace/:id2 [delete]
func (o *CollaborativeAreaController) RemoveWorkspace() {
	caller := tools.NewHTTPCaller(paths) // caller used to send to peers
	caller.Disabled = oclib.IsQueryParamsEquals(o.Ctx.Input, "is_remote", true)
	id := o.Ctx.Input.Param(":id")
	id2 := o.Ctx.Input.Param(":id2")
	r := oclib.LoadOne(oclib.LibDataEnum(tools.COLLABORATIVE_AREA), id)
	shared := r.ToCollaborativeArea()
	newWorkspace := []string{}
	if slices.Contains(shared.Workspaces, id2) {
		for _, w := range shared.Workspaces {
			if w != id2 {
				newWorkspace = append(newWorkspace, w)
			}
		}
		shared.Workspaces = newWorkspace
	}
	o.Data["json"] = oclib.UpdateOne(oclib.LibDataEnum(tools.COLLABORATIVE_AREA), shared.Serialize(), id, caller)
	o.ServeJSON()
}

// @Title Remove Workflow
// @Description find shared workspace by id
// @Param	id		path 	string	true		"the id you want to get"
// @Param	id2		path 	string	true		"the id you want to add"
// @Success 200 {shared workspace} models.shared_workspace
// @router /:id/workflow/:id2 [delete]
func (o *CollaborativeAreaController) RemoveWorkflow() {
	caller := tools.NewHTTPCaller(paths) // caller used to send to peers
	caller.Disabled = oclib.IsQueryParamsEquals(o.Ctx.Input, "is_remote", true)
	id := o.Ctx.Input.Param(":id")
	id2 := o.Ctx.Input.Param(":id2")
	r := oclib.LoadOne(oclib.LibDataEnum(tools.COLLABORATIVE_AREA), id)
	fmt.Println("RemoveWorkflow", r)
	shared := r.ToCollaborativeArea()
	newWorkflows := []string{}
	if slices.Contains(shared.Workflows, id2) {
		for _, w := range shared.Workflows {
			if w != id2 {
				newWorkflows = append(newWorkflows, w)
			}
		}
		shared.Workflows = newWorkflows
	}
	o.Data["json"] = oclib.UpdateOne(oclib.LibDataEnum(tools.COLLABORATIVE_AREA), shared.Serialize(), id, caller)
	o.ServeJSON()
}

// @Title Remove Peer
// @Description find shared workspace by id
// @Param	id		path 	string	true		"the id you want to get"
// @Param	id2		path 	string	true		"the id you want to add"
// @Success 200 {shared workspace} models.shared_workspace
// @router /:id/peer/:id2 [delete]
func (o *CollaborativeAreaController) RemovePeer() {
	caller := tools.NewHTTPCaller(paths) // caller used to send to peers
	caller.Disabled = oclib.IsQueryParamsEquals(o.Ctx.Input, "is_remote", true)
	id := o.Ctx.Input.Param(":id")
	id2 := o.Ctx.Input.Param(":id2")
	r := oclib.LoadOne(oclib.LibDataEnum(tools.COLLABORATIVE_AREA), id)
	shared := r.ToCollaborativeArea()
	newPeers := []string{}
	if shared.CreatorID != id2 {
		o.Data["json"] = map[string]interface{}{
			"data":  nil,
			"code":  409,
			"error": "You can't remove the creator from the shared workspace",
		}
		o.ServeJSON()
		return
	}
	if slices.Contains(shared.Peers, id) && shared.CreatorID != id2 {
		for _, peer := range shared.Peers {
			if peer != id2 {
				newPeers = append(newPeers, peer)
			}
		}
		shared.Peers = newPeers
	}
	o.Data["json"] = oclib.UpdateOne(oclib.LibDataEnum(tools.COLLABORATIVE_AREA), shared.Serialize(), id, caller)
	o.ServeJSON()
}

// @Title Remove Rule
// @Description find shared workspace by id
// @Param	id		path 	string	true		"the id you want to get"
// @Param	id2		path 	string	true		"the id you want to add"
// @Success 200 {shared workspace} models.shared_workspace
// @router /:id/rule/:id2 [delete]
func (o *CollaborativeAreaController) RemoveRule() {
	caller := tools.NewHTTPCaller(paths) // caller used to send to peers
	caller.Disabled = oclib.IsQueryParamsEquals(o.Ctx.Input, "is_remote", true)
	id := o.Ctx.Input.Param(":id")
	id2 := o.Ctx.Input.Param(":id2")
	r := oclib.LoadOne(oclib.LibDataEnum(tools.COLLABORATIVE_AREA), id)
	shared := r.ToCollaborativeArea()
	newRules := []string{}
	if shared != nil && slices.Contains(shared.Rules, id2) {
		for _, rule := range shared.Rules {
			if rule != id2 {
				newRules = append(newRules, rule)
			}
		}
		shared.Rules = newRules
	}
	o.Data["json"] = oclib.UpdateOne(oclib.LibDataEnum(tools.COLLABORATIVE_AREA), shared.Serialize(), id, caller)
	o.ServeJSON()
}

// @Title Add Workspace
// @Description find shared workspace by id
// @Param	id		path 	string	true		"the id you want to get"
// @Param	id2		path 	string	true		"the id you want to add"
// @Success 200 {shared workspace} models.shared_workspace
// @router /:id/workspace/:id2 [post]
func (o *CollaborativeAreaController) AddWorkspace() {
	caller := tools.NewHTTPCaller(paths) // caller used to send to peers
	caller.Disabled = oclib.IsQueryParamsEquals(o.Ctx.Input, "is_remote", true)
	id := o.Ctx.Input.Param(":id")
	id2 := o.Ctx.Input.Param(":id2")
	r := oclib.LoadOne(oclib.LibDataEnum(tools.COLLABORATIVE_AREA), id)
	if r.Code != 200 {
		o.Data["json"] = r
		o.ServeJSON()
		return
	}
	shared := r.ToCollaborativeArea()
	if shared != nil && !slices.Contains(shared.Workspaces, id2) {
		shared.Workspaces = append(shared.Workspaces, id2)
	}
	o.Data["json"] = oclib.UpdateOne(oclib.LibDataEnum(tools.COLLABORATIVE_AREA), shared.Serialize(), id, caller)
	o.ServeJSON()
}

// @Title Add Workflow
// @Description find shared workspace by id
// @Param	id		path 	string	true		"the id you want to get"
// @Param	id2		path 	string	true		"the id you want to add"
// @Success 200 {shared workspace} models.shared_workspace
// @router /:id/workflow/:id2 [post]
func (o *CollaborativeAreaController) AddWorkflow() {
	caller := tools.NewHTTPCaller(paths) // caller used to send to peers
	caller.Disabled = oclib.IsQueryParamsEquals(o.Ctx.Input, "is_remote", true)
	id := o.Ctx.Input.Param(":id")
	id2 := o.Ctx.Input.Param(":id2")
	r := oclib.LoadOne(oclib.LibDataEnum(tools.COLLABORATIVE_AREA), id)
	shared := r.ToCollaborativeArea()
	if shared != nil && !slices.Contains(shared.Workflows, id2) {
		shared.Workflows = append(shared.Workflows, id2)
	}
	o.Data["json"] = oclib.UpdateOne(oclib.LibDataEnum(tools.COLLABORATIVE_AREA), shared.Serialize(), id, caller)
	o.ServeJSON()
}

// @Title Add Peer
// @Description find shared workspace by id
// @Param	id		path 	string	true		"the id you want to get"
// @Param	id2		path 	string	true		"the id you want to add"
// @Success 200 {shared workspace} models.shared_workspace
// @router /:id/peer/:id2 [post]
func (o *CollaborativeAreaController) AddPeer() {
	caller := tools.NewHTTPCaller(paths) // caller used to send to peers
	caller.Disabled = oclib.IsQueryParamsEquals(o.Ctx.Input, "is_remote", true)
	id := o.Ctx.Input.Param(":id")
	id2 := o.Ctx.Input.Param(":id2")
	r := oclib.LoadOne(oclib.LibDataEnum(tools.COLLABORATIVE_AREA), id)
	shared := r.ToCollaborativeArea()
	if shared != nil && !slices.Contains(shared.Peers, id2) {
		shared.Peers = append(shared.Peers, id2)
	}
	o.Data["json"] = oclib.UpdateOne(oclib.LibDataEnum(tools.COLLABORATIVE_AREA), shared.Serialize(), id, caller)
	o.ServeJSON()
}

// @Title Add Rule
// @Description find shared workspace by id
// @Param	id		path 	string	true		"the id you want to get"
// @Param	id2		path 	string	true		"the id you want to add"
// @Success 200 {shared workspace} models.shared_workspace
// @router /:id/rule/:id2 [post]
func (o *CollaborativeAreaController) AddRule() {
	caller := tools.NewHTTPCaller(paths) // caller used to send to peers
	caller.Disabled = oclib.IsQueryParamsEquals(o.Ctx.Input, "is_remote", true)
	id := o.Ctx.Input.Param(":id")
	id2 := o.Ctx.Input.Param(":id2")
	r := oclib.LoadOne(oclib.LibDataEnum(tools.COLLABORATIVE_AREA), id)
	shared := r.ToCollaborativeArea()
	if shared != nil && !slices.Contains(shared.Rules, id2) {
		shared.Rules = append(shared.Rules, id2)
	}
	o.Data["json"] = oclib.UpdateOne(oclib.LibDataEnum(tools.COLLABORATIVE_AREA), shared.Serialize(), id, caller)
	o.ServeJSON()
}

// @Title Delete
// @Description delete the shared workspace
// @Param	id		path 	string	true		"The id you want to delete"
// @Success 200 {shared workspace} delete success!
// @router /:id [delete]
func (o *CollaborativeAreaController) Delete() {
	id := o.Ctx.Input.Param(":id")
	caller := tools.NewHTTPCaller(paths) // caller used to send to peers
	caller.Disabled = oclib.IsQueryParamsEquals(o.Ctx.Input, "is_remote", true)
	o.Data["json"] = oclib.DeleteOne(oclib.LibDataEnum(tools.COLLABORATIVE_AREA), id, caller)
	o.ServeJSON()
}
